from dataclasses import Field, dataclass, field
from datetime import datetime
from typing import Generic, TypeVar, Optional, List

from mag_tools.exception.app_exception import AppException
from mag_tools.model.service_status import ServiceStatus

T = TypeVar('T')

@dataclass
class Results(Generic[T]):
    code: Optional[ServiceStatus] = None
    message: Optional[str] = None
    data: Optional[List[T]] = field(default_factory=list)
    total_count: Optional[int] = None
    timestamp: Optional[datetime] = field(default_factory=datetime.now)

    @staticmethod
    def exception(ex: Exception):
        message = str(ex) if ex.args else str(ex.__cause__)
        return Results(code=ServiceStatus.INTERNAL_SERVER_ERROR, message=message)

    @staticmethod
    def success(data: Optional[List[T]] = None):
        return Results(message="OK", data=data)

    @staticmethod
    def fail(message: str):
        return Results(message=message)

    @property
    def is_success(self) -> bool:
        return self.code == ServiceStatus.OK.code

    @property
    def size(self) -> int:
        return len(self.data)

    @property
    def first(self) -> Optional[T]:
        return self.data[0] if self.data and len(self.data) > 0 else None

    def check(self) -> None:
        if not self.is_success:
            raise AppException(self.message)

    def get(self, idx: int) -> Optional[T]:
        self.check()
        return self.data[idx] if idx < self.size else None

    def to_dict(self):
        return {
            'code': self.code.code,
            'message': self.message,
            'timestamp': self.timestamp,
            'data': self.data,
            'total_count': self.total_count
        }


import re
import subprocess
from dataclasses import dataclass
from typing import Optional

import wmi

from mag_tools.bean.sys.operate_system import OperateSystem
from mag_tools.model.memory_type import MemoryType

@dataclass
class MemoryModule:
    memory_type: Optional[MemoryType] = None  # 内存类型
    capacity: Optional[int] = None            # 容量 (GB)
    frequency: Optional[int] = None           # 频率 (MHz)
    voltage: Optional[float] = None           # 电压 (V)
    form_factor: Optional[str] = None         # 形态
    used_slots: Optional[int] = None          # 使用的插槽数
    speed: Optional[int] = None               # 速度 (MB/s)
    latency: Optional[int] = None             # 延迟 (CL)
    serial_number: Optional[str] = None       # 序列号

    @classmethod
    def get_modules(cls):
        if OperateSystem.is_windows():
            return cls.__get_from_windows()
        else:
            return cls.__get_from_linux()

    def __str__(self):
        """
        返回内存条的字符串表示
        """
        return f"MemoryModule(memory_type='{self.memory_type}', capacity={self.capacity} GB, frequency={self.frequency} MHz, voltage={self.voltage} V, latency={self.latency} CL, speed={self.speed} MB/s, form_factor='{self.form_factor}', used_slots={self.used_slots}, serial_number='{self.serial_number}')"

    @classmethod
    def __get_from_windows(cls):
        modules = []
        c = wmi.WMI()
        for memory in c.Win32_PhysicalMemory():
            memory_type = MemoryType.of_code(memory.MemoryType)
            capacity = int(memory.Capacity) // (1024 ** 3)  # 将字节转换为GB
            frequency = memory.Speed
            voltage = memory.ConfiguredVoltage / 1000.0  # 将毫伏转换为伏特
            form_factor = memory.FormFactor
            serial_number = memory.SerialNumber

            modules.append(cls(memory_type=memory_type, capacity=capacity, frequency=frequency, voltage=voltage,
                               form_factor=form_factor, serial_number=serial_number))

        return modules

    @classmethod
    def __get_from_linux(cls):
        """
        获取当前系统的所有内存条信息，并返回一个包含多个Memory实例的列表
        """
        result = subprocess.run(['sudo', 'dmidecode', '--type', 'memory'], stdout=subprocess.PIPE)
        output = result.stdout.decode()

        modules = []
        for section in output.split('\n\n'):
            memory_type = capacity = frequency = voltage = form_factor = serial_number = None

            for line in section.split('\n'):
                if 'Type:' in line and 'Type Detail:' not in line:
                    memory_type = line.split(':')[-1].strip()
                elif 'Size:' in line and 'No Module Installed' not in line:
                    capacity = int(re.findall(r'\d+', line.split(':')[-1].strip())[0])
                elif 'Speed:' in line:
                    frequency = int(re.findall(r'\d+', line.split(':')[-1].strip())[0])
                elif 'Voltage:' in line:
                    voltage = float(re.findall(r'\d+\.\d+', line.split(':')[-1].strip())[0])
                elif 'Form Factor:' in line:
                    form_factor = line.split(':')[-1].strip()
                elif 'Serial Number:' in line:
                    serial_number = line.split(':')[-1].strip()

            if memory_type and capacity:
                modules.append(MemoryModule(memory_type=memory_type, capacity=capacity, frequency=frequency, voltage=voltage,
                                            form_factor=form_factor, serial_number=serial_number))

        return modules

# 示例调用
if __name__ == "__main__":
    modules = MemoryModule.get_modules()
    for module in modules:
        print(module)



from dataclasses import dataclass
from typing import List, Optional

import wmi

from mag_tools.model.disk_type import DiskType
from mag_tools.model.fs_type import FsType

@dataclass
class DiskPartition:
    fs_type: Optional[FsType] = None
    mount_point: Optional[str] = None       # 驱动装载点

@dataclass
class Disk:
    """
    磁盘参数类
    """
    serial_number: Optional[str] = None   # 序列号
    disk_type: Optional[DiskType] = None # 磁盘类型
    model: Optional[str] = None
    media_type: Optional[str] = None
    manufacturer: Optional[str] = None
    capacity: Optional[int] = None    # 总容量，单位为G

    @classmethod
    def get_info(cls):
        physical_disks = []
        c = wmi.WMI()
        for disk in c.Win32_DiskDrive():
            manufacturer, dick_type = cls.__parse(disk.Model)

            info = cls(serial_number=disk.DeviceID,
                       disk_type=dick_type,
                       model=disk.Model,
                       capacity=int(disk.Size) / 1000**3,
                       media_type=disk.MediaType,
                       manufacturer=manufacturer)
            physical_disks.append(info)

        return physical_disks

    def __str__(self):
        """
        返回磁盘参数的字符串表示
        """
        attributes = [f"{attr.replace('_', ' ').title()}: {getattr(self, attr)}" for attr in vars(self) if
                      getattr(self, attr) is not None]
        return ", ".join(attributes)

    @classmethod
    def __parse(cls, model):
        items = model.split()
        manufacturer = items[0] if len(items) > 0 else None
        dick_type = DiskType.of_code(items[1]) if len(items) > 1 else None

        return manufacturer, dick_type


import json
from decimal import Decimal
from datetime import datetime, date
from enum import Enum
from typing import Optional, Type, TypeVar, List, Dict, Any

from anytree import Node
from mag_tools.bean.easy_map import K, V
from mag_tools.jsonparser.datetim_encoder import DateTimeEncoder

T = TypeVar('T')
E = TypeVar('E')

class JsonParser:
    @staticmethod
    def to_string(json_src: str) -> str:
        return json.loads(json_src)

    @staticmethod
    def to_decimal(json_src: str) -> Optional[Decimal]:
        if not json_src:
            return None
        return Decimal(json.loads(json_src))

    @staticmethod
    def to_float(json_src: str) -> Optional[float]:
        if not json_src:
            return None
        return float(json.loads(json_src))

    @staticmethod
    def to_int(json_src: str) -> Optional[int]:
        if not json_src:
            return None
        return int(json.loads(json_src))

    @staticmethod
    def to_datetime(json_src: str) -> Optional[datetime]:
        if not json_src:
            return None
        return datetime.strptime(json.loads(json_src), '%Y-%m-%dT%H:%M:%S.%fZ')

    @staticmethod
    def to_bool(json_src: str) -> bool:
        json_str = JsonParser.to_string(json_src).lower()
        return json_str in ("y", "yes", "true", "t", "1")

    @staticmethod
    def to_enum(json_src: str, enum_type: Type[E]) -> Optional[E]:
        if not json_src or not issubclass(enum_type, Enum):
            return None
        enum_str = JsonParser.to_string(json_src)
        for member in enum_type:
            if member.name == enum_str or member.value == enum_str:
                return member
        return None

    @staticmethod
    def to_bean(json_src: str, clazz: Type[T]) -> Optional[T]:
        if not json_src or not clazz:
            return None
        return json.loads(json_src, object_hook=lambda d: clazz(**d))

    @staticmethod
    def to_list(json_src: str, clazz: Type[T]) -> List[T]:
        if not json_src:
            return []
        return json.loads(json_src, object_hook=lambda d: clazz(**d))

    @staticmethod
    def to_map(json_src: str, key_cls: Type[K], value_cls: Type[V]) -> Dict[K, V]:
        if not json_src:
            return {}
        data_map = json.loads(json_src)
        return {key_cls(key): value_cls(**value) if isinstance(value, dict) else value_cls(value) for key, value in data_map.items()}

    @staticmethod
    def to_easy_map(json_src: str, value_cls: Type[V]) -> Dict[str, V]:
        return JsonParser.to_map(json_src, str, value_cls)


    @staticmethod
    def to_tree(json_src: str, clazz: Type[T]) -> Node:
        if not json_src:
            return Node("root")

        data = json.loads(json_src)

        def create_node(data_, parent=None):
            if isinstance(data_, dict):
                name = data_.get('name', 'root')
                data_object = clazz(**data_) if clazz != str else name
                node = Node(name, parent=parent, data=data_object)
                for child_data in data_.get('children', []):
                    create_node(child_data, parent=node)
            else:
                node = Node(str(data_), parent=parent)
            return node

        tree = create_node(data)
        return tree

    @staticmethod
    def from_bean(obj: Any) -> Optional[str]:
        if isinstance(obj, (int, float, Decimal, bool, str)):
            return json.dumps(obj) if obj else None
        elif isinstance(obj, (datetime, date)):
            return obj.isoformat() if obj else None
        elif isinstance(obj, list):
            return json.dumps(obj, default=lambda o: o.__dict__) if obj else "[]"
        elif isinstance(obj, dict):
            return json.dumps(obj, default=lambda o: o.__dict__) if obj else "{}"
        elif hasattr(obj, 'to_dict'):
            return json.dumps(obj.to_dict(), cls=DateTimeEncoder) if obj else None
        else:
            return json.dumps(obj, default=lambda o: o.__dict__, cls=DateTimeEncoder) if obj else None

    @staticmethod
    def to_json_by_type(obj: Any, obj_type: Type) -> Optional[str]:
        if obj is None:
            return None
        return json.dumps(obj, default=lambda o: o.__dict__, sort_keys=True, indent=4)



from json import JSONEncoder

from enum import Enum


class EnumEncoder(JSONEncoder):
    def default(self, obj):
        if isinstance(obj, Enum):
            return obj.name
        return super().default(obj)



from mag_tools.model.base_enum import BaseEnum


class ComputerType(BaseEnum):
    """
    计算机类型枚举类
    """
    DESKTOP = ("Desktop", "台式计算机")
    LAPTOP = ("Laptop", "笔记本电脑")
    SERVER = ("Server", "服务器")
    WORKSTATION = ("Workstation", "工作站")
    TABLET = ("Tablet", "平板电脑")